import django


def pytest_configure(config):
    from django.conf import settings

    settings.configure(
        DEBUG_PROPAGATE_EXCEPTIONS=True,
        DEBUG=True,
        INSTALLED_APPS=[
            "django_filtering_ui",
        ],
        TEMPLATES = [
            {
                "BACKEND": "django.template.backends.django.DjangoTemplates",
                "APP_DIRS": True,
            },
        ],
        STATIC_URL = "/static/"
    )

    django.setup()



from urllib.parse import urljoin
from django.conf import settings


DJANGO_FILTERING_UI_DEV_ENABLED = False
DJANGO_FILTERING_UI_DEV_URL = "http://localhost:5173"
DJANGO_FILTERING_UI_DEV_PATH = "/js-src/"


def is_dev_enabled() -> bool:
    return getattr(settings, "DJANGO_FILTERING_UI_DEV_ENABLED", DJANGO_FILTERING_UI_DEV_ENABLED)

def get_dev_url() -> str:
    url = getattr(settings, "DJANGO_FILTERING_UI_DEV_URL", DJANGO_FILTERING_UI_DEV_URL)
    path = getattr(settings, "DJANGO_FILTERING_UI_DEV_PATH", DJANGO_FILTERING_UI_DEV_PATH)
    return urljoin(url, path)



from typing import Any
from urllib.parse import urljoin

from django import template
from django.conf import settings
from django.templatetags.static import static
from django.utils.html import escapejs, mark_safe
from django.utils.safestring import SafeString

from ..conf import get_dev_url, is_dev_enabled


register = template.Library()


@register.simple_tag()
def entrypoint(name: str) -> SafeString:
    """
    Renders a ``<script>`` tag with the static resource path
    or a url to the development server path.
    The ``name`` parameter matches the input name defined in ``vite.config.js``
    (i.e. either filtering or listing).

    The development server url is used when ``settings.DJANGO_FILTERING_UI_DEV`` is defined.
    When using a development server the resulting script tag will
    contain the ``crossorigin`` attribute to provide CORS support.
    """
    opts = ''
    if is_dev_enabled():
        base_uri = get_dev_url()
        uri = urljoin(base_uri, name)
        opts = ' crossorigin'
    else:
        uri = static(f"django-filtering-ui/{name}")
    return mark_safe(
        f'<script type="module" src="{uri}"{opts}></script>'
    )


@register.simple_tag()
def vue_provide(key: str, value: Any, quote: bool = True) -> SafeString:
    """
    Writes a ``<script>`` tag with javascript that will be picked up by the
    ``vue-plugin-django-utils`` VueJS plugin package on the frontend.
    The provided key and value is essentially like calling VueJS'
    ``provide(key, value)`` function.

    The key value pair can be used in VueJS with ``inject('key-name')``.

    Be sure to set ``quote=False`` if supplying JSON encoded data.
    """
    if quote:
        value = escapejs(value)
        q = '"'
    else:
        q = ''
    return mark_safe(
        "<script>window.vueProvided = window.vueProvided || "
        f'{{}}; vueProvided["{key}"] = {q}{value}{q};</script>'
    )

@register.inclusion_tag("django_filtering_ui/inclusion.html", takes_context=True)
def django_filtering_ui(context):
    """
    This template tag encapsulates the entrypoint and vue provides
    templating into a single call.

    Renders all the tags needed for the given ``entrypoint``
    This will include the ``<script>`` tag for the entrypoint
    and any supporting vue provides.

    Needed context is:
        - 'entrypoint' - either 'listing' or 'filtering'
        - 'listing_url' - a url to the listing page
        - 'filtering_options_schema' - the options schema
        - 'filtering_json_schema' - the jsonschema

    """
    ep = context.get('entrypoint')
    if ep not in ('filtering', 'listing'):
        raise ValueError(f"Invalid 'entrypoint' value:  {ep}")

    entrypoint_name = f"{ep}.js"
    local_context = {
        'entrypoint': entrypoint_name,
        'filtering_url': context['filtering_url'],
        'listing_url': context['listing_url'],
        'filtering_options_schema': context['filtering_options_schema'],
        'filtering_json_schema': context['filtering_json_schema'],
        'DEBUG': "true" if settings.DEBUG else "false",  # js true/false value
    }
    return local_context



from urllib.parse import urljoin

import pytest
from django.template import Template, Context
from django.templatetags.static import static
from django.utils.html import escapejs
from django_filtering_ui.conf import (
    DJANGO_FILTERING_UI_DEV_PATH,
    DJANGO_FILTERING_UI_DEV_URL,
)

from django_filtering_ui.templatetags.django_filtering_ui import (
    django_filtering_ui,
    entrypoint,
    vue_provide,
)


class TestEntrypoint:
    """
    Tests the ``entrypoint`` templatetag.
    """

    def test_default_use(self):
        name = 'filtering.js'
        rendered = entrypoint(name)

        url = static(f'django-filtering-ui/{name}')
        expected_result = f'<script type="module" src="{url}"></script>'
        assert rendered.strip() == expected_result

    def test_dev_use(self, settings):
        settings.DJANGO_FILTERING_UI_DEV_ENABLED = True
        name = 'filtering.js'
        rendered = entrypoint(name)

        url = urljoin(urljoin(DJANGO_FILTERING_UI_DEV_URL, DJANGO_FILTERING_UI_DEV_PATH), name)
        expected_result = f'<script type="module" src="{url}" crossorigin></script>'
        assert rendered.strip() == expected_result


class TestVueProvide:
    """
    Tests the ``vue_provide`` templatetag.
    """

    def test_default_use(self):
        key = 'msg'
        value = "testing message with \"quoted\" content"
        rendered = vue_provide(key, value)

        expected_result = (
            "<script>window.vueProvided = window.vueProvided || {}; "
            f'vueProvided["{key}"] = "{escapejs(value)}";</script>'
        )
        assert rendered.strip() == expected_result

    def test_unquoted_use(self, settings):
        key = 'notices'
        value = '{"notices": ["testing message"]}'
        rendered = vue_provide(key, value, quote=False)

        expected_result = (
            "<script>window.vueProvided = window.vueProvided || {}; "
            # Note, the missing quotes around the value.
            f'vueProvided["{key}"] = {value};</script>'
        )
        assert rendered.strip() == expected_result


class TestDjangoFilteringUi:
    """
    Tests the ``django_filtering_ui`` templatetag.
    """

    def test_default_use(self):
        entrypoint_name = 'filtering'
        context = {
            'listing_url': '$$$listing$$$',
            'filtering_url': '$$$filtering$$$',
            'filtering_options_schema': '$$$filtering_options_schema$$$',
            'filtering_json_schema': '$$$filtering_json_schema$$$',
        }

        tmplt = Template(
            "{% load django_filtering_ui from django_filtering_ui %}"
            f"{{% with entrypoint='{entrypoint_name}' %}}"
            "{% django_filtering_ui %}"
            "{% endwith %}"
        )
        rendered = tmplt.render(Context(context))
        assert 'src="/static/django-filtering-ui/filtering.js"' in rendered
        assert 'vueProvided["model-listing-url"] = "$$$listing$$$";' in rendered
        assert 'vueProvided["model-filtering-url"] = "$$$filtering$$$";' in rendered
        assert 'vueProvided["filtering-options-schema"] = $$$filtering_options_schema$$$;' in rendered
        assert 'vueProvided["filtering-json-schema"] = $$$filtering_json_schema$$$;' in rendered
        assert 'vueProvided["debug-enabled"] = false;' in rendered

    def test_invalid_use(self):
        entrypoint_name = 'bogus'
        context = {
            'entrypoint': entrypoint_name,
            'listing_url': '$$$listing$$$',
            'filtering_url': '$$$filtering$$$',
            'filtering_options_schema': '$$$filtering_options_schema$$$',
            'filtering_json_schema': '$$$filtering_json_schema$$$',
        }

        with pytest.raises(ValueError) as caught:
            django_filtering_ui(context)

        assert "Invalid 'entrypoint' value" in caught.value.args[0]
